'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _slicedToArray = _interopDefault(require('@babel/runtime/helpers/slicedToArray'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var redux = require('redux');
var reactRedux = require('react-redux');

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
/**
 * Utils
 */

var ERR = {
  NOT_STRING: function NOT_STRING(name) {
    return "'".concat(name, "' must be a string");
  },
  NOT_ARRAY: function NOT_ARRAY(name) {
    return "'".concat(name, "' must be an array");
  },
  NOT_OBJECT: function NOT_OBJECT(name) {
    return "'".concat(name, "' must be an object");
  },
  NOT_CLASS: function NOT_CLASS(name) {
    return "'".concat(name, "' must be a class");
  },
  NO_DISPATCH: function NO_DISPATCH() {
    return "Please don't use 'dispatch' directly in Retalk";
  },
  NOT_EXIST: function NOT_EXIST(name) {
    return "'".concat(name, "' Model dose not exist");
  }
};

var isObject = function isObject(obj) {
  return _typeof(obj) === 'object' && obj !== null && !Array.isArray(obj);
};
/**
 * Add a model, create the model's reducer
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} models
 * @param {object} reducers
 */


var createReducer = function createReducer(name, Model, models, reducers) {
  var model = new Model();
  models[name] = model;

  reducers[name] = function () {
    var currentState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : model.state || null;
    var action = arguments.length > 1 ? arguments[1] : undefined;

    var _action$type$split = action.type.split('/'),
        _action$type$split2 = _slicedToArray(_action$type$split, 1),
        modelName = _action$type$split2[0];

    if (modelName === name) return _objectSpread({}, currentState, {}, action.payload);
    return currentState;
  };
};
/**
 * Format a model's actions
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} models
 * @param {function} newDispatch
 * @param {function} realDispatch
 */


var createActions = function createActions(name, Model, models, newDispatch, realDispatch) {
  var model__proto__ = Model.prototype;
  model__proto__.models = models;
  var model = models[name];
  newDispatch[name] = {};
  var setStateMap = {};
  Object.getOwnPropertyNames(model__proto__).forEach(function (actionName) {
    if (actionName === 'constructor' || actionName === 'models') return;

    setStateMap[actionName] = function reducer(payload) {
      model.state = _objectSpread({}, model.state, {}, payload);
      return realDispatch({
        type: "".concat(name, "/").concat(actionName),
        payload: payload
      });
    };

    var setLoading = function setLoading(type, loading) {
      model__proto__[actionName].loading = loading;
      return realDispatch({
        type: "".concat(name, "/").concat(actionName, "/").concat(type, "_LOADING"),
        payload: {
          loading: loading
        }
      });
    };

    var oldAction = model__proto__[actionName].bind(model);

    var newAction = function action() {
      model__proto__.setState = setStateMap[actionName];
      var result = oldAction.apply(void 0, arguments);

      if (!result || typeof result.then !== 'function') {
        delete model__proto__.setState;
        return result;
      }

      return new Promise(function (resolve, reject) {
        setLoading('START', true);
        result.then(resolve).catch(reject).finally(function () {
          setLoading('STOP', false);
          delete model__proto__.setState;
        });
      });
    };

    newDispatch[name][actionName] = newAction;
    model__proto__[actionName] = newAction;
  });
};
/**
 * Initialize all models
 *
 * @param {object} initialModels
 * @param {array} middleware
 * @return {object} Store
 */


exports.setStore = function setStore() {
  var initialModels = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var middleware = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var __DEV__ = process.env.NODE_ENV !== 'production';

  if (__DEV__) {
    if (!isObject(initialModels)) throw new Error(ERR.NOT_OBJECT('initialModels'));
    if (!Array.isArray(middleware)) throw new Error(ERR.NOT_ARRAY('middleware'));
  }

  var models = {};
  var reducers = {};
  var initialModelList = Object.entries(initialModels); // Reducers

  if (__DEV__) {
    initialModelList.forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          name = _ref2[0],
          Model = _ref2[1];

      if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
      createReducer(name, Model, models, reducers);
    });
  } else {
    initialModelList.forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          name = _ref4[0],
          Model = _ref4[1];

      createReducer(name, Model, models, reducers);
    });
  } // Store


  var __COMPOSE__ = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__;
  var newCompose = __DEV__ && __COMPOSE__ ? __COMPOSE__ : redux.compose;
  var store = redux.createStore(redux.combineReducers(reducers), newCompose(redux.applyMiddleware.apply(void 0, _toConsumableArray(middleware))));
  var realDispatch = store.dispatch;

  var newDispatch = function newDispatch() {
    throw new Error(ERR.NO_DISPATCH());
  };

  store.dispatch = newDispatch; // Actions

  initialModelList.forEach(function (_ref5) {
    var _ref6 = _slicedToArray(_ref5, 2),
        name = _ref6[0],
        Model = _ref6[1];

    createActions(name, Model, models, newDispatch, realDispatch);
  }); // store.add

  store.add = function (initialModelsToAdd) {
    if (__DEV__) {
      if (!isObject(initialModelsToAdd)) throw new Error(ERR.NOT_OBJECT('initialModels'));
      Object.entries(initialModelsToAdd).forEach(function (_ref7) {
        var _ref8 = _slicedToArray(_ref7, 2),
            name = _ref8[0],
            Model = _ref8[1];

        if (name in models) return;
        if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
        createReducer(name, Model, models, reducers);
        store.replaceReducer(redux.combineReducers(reducers));
        createActions(name, Model, models, newDispatch, realDispatch);
      });
    } else {
      Object.entries(initialModelsToAdd).forEach(function (_ref9) {
        var _ref10 = _slicedToArray(_ref9, 2),
            name = _ref10[0],
            Model = _ref10[1];

        if (name in models) return;
        createReducer(name, Model, models, reducers);
        store.replaceReducer(redux.combineReducers(reducers));
        createActions(name, Model, models, newDispatch, realDispatch);
      });
    }
  };

  return store;
};
/**
 * Eject some models to a component
 *
 * @param {string[]} names
 * @return {function} Higher-order component
 */


exports.withStore = function withStore() {
  for (var _len = arguments.length, names = new Array(_len), _key = 0; _key < _len; _key++) {
    names[_key] = arguments[_key];
  }

  if (typeof names[0] !== 'string') {
    return reactRedux.connect.apply(void 0, names);
  }

  var __DEV__ = process.env.NODE_ENV !== 'production';

  var mapStore = function mapStore(store) {
    return names.reduce(function (merged, name) {
      return _objectSpread({}, merged, {}, store[name]);
    }, {});
  };

  var mapState;

  if (__DEV__) {
    mapState = function mapState(state) {
      return names.reduce(function (merged, name) {
        if (typeof name !== 'string') throw new Error(ERR.NOT_STRING('name'));
        var partState = state[name];
        if (typeof partState === 'undefined') throw new Error(ERR.NOT_EXIST(name));
        return _objectSpread({}, merged, {}, partState);
      }, {});
    };
  } else {
    mapState = mapStore;
  }

  return reactRedux.connect(mapState, mapStore);
};

Object.defineProperty(exports, 'Provider', {
  enumerable: true,
  get: function () {
    return reactRedux.Provider;
  }
});
